using System;
using System.Collections;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Text;
using System.IO;
using System.Xml;
using System.Windows.Forms;
using MediaPortal.GUI.Library;
using MediaPortal.Mixer;

namespace WINLIRC_PLUGIN
{
    class StateObject
    {
        // Client socket.
        public Socket workSocket = null;
        // Size of receive buffer.
        public const int BufferSize = 256;
        // Receive buffer.
        public byte[] buffer = new byte[BufferSize];
        // Received data string.
        public StringBuilder sb = new StringBuilder();
    }
    
    class Connection
    {
        
        public static Settings wnd; //Setting Window
        public static bool configmode = false;
        private const int port = 8765;

        
        public static ArrayList IRcommands = new ArrayList(); //captured IR Commands
        public static Array MPcommands; //MediaPortal Commands
        public static ArrayList CommandTable = new ArrayList(); //stores the MPcommands for all IRcommands
        public static XmlDocument doc = new XmlDocument(); //XML document for settings and commands

        // ManualResetEvent instances signal completion.
        private static ManualResetEvent connectDone = new ManualResetEvent(false);
        private static ManualResetEvent sendDone = new ManualResetEvent(false);
        private static ManualResetEvent receiveDone = new ManualResetEvent(false);

        // The response from the remote device.
        private static String response = String.Empty;



        public static void connect()
        {
            try
            {
                
                IPEndPoint remoteEP = new IPEndPoint(IPAddress.Parse("127.0.0.1"), port);

                // Create a TCP/IP socket.
                Socket client = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

                // Connect to the remote endpoint.
                client.BeginConnect(remoteEP, new AsyncCallback(ConnectCallback), client);
                connectDone.WaitOne();

                
                // Send test data to the remote device.
                //Send(client, "This is a test<EOF>");
                //sendDone.WaitOne();
                
                // Receive the response from the remote device.
                Receive(client);
                //receiveDone.WaitOne();

                // Write the response to the console.
                
                LoadMPCommands();
                //Load the XML file
                doc.Load(Application.ExecutablePath.Remove(Application.ExecutablePath.LastIndexOf("\\")).ToString() + "\\plugins\\process\\WinLIRC_Plugin.xml");
                Logging.Log("MP is connected to WInLIRC");
                // Release the socket.
                //client.Shutdown(SocketShutdown.Both);
                //client.Close();
                
            }
            catch 
            {
                Logging.Log("MP is not connected to WInLIRC");
                try
                {
                    LoadMPCommands();
                    doc.Load(Application.ExecutablePath.Remove(Application.ExecutablePath.LastIndexOf("\\")).ToString() + "\\plugins\\process\\WinLIRC_Plugin.xml");
                    
                }
                catch
                {
                    Logging.Log("Coul not load xml file");
                }
                
            }
        }

        private static void ConnectCallback(IAsyncResult ar)
        {
            try
            {
                // Retrieve the socket from the state object.
                Socket client = (Socket)ar.AsyncState;

                // Complete the connection.
                client.EndConnect(ar);

                // Signal that the connection has been made.
                connectDone.Set();
                Logging.Log("Connection established on Port " + port.ToString());
            }
            catch 
            {
              
            }
        }

        private static void Receive(Socket client)
        {
            try
            {
                // Create the state object.
                StateObject state = new StateObject();
                state.workSocket = client;
                
                // Begin receiving the data from the remote device.
                client.BeginReceive(state.buffer, 0, StateObject.BufferSize, 0, new AsyncCallback(ReceiveCallback), state);
            }
            catch 
            {
                
            }
        }

        private static void ReceiveCallback(IAsyncResult ar)
        {
            try
            {
                // Retrieve the state object and the client socket 
                // from the asynchronous state object.
                StateObject state = (StateObject)ar.AsyncState;
                Socket client = state.workSocket;
                
                // Read data from the remote device.
                int bytesRead = client.EndReceive(ar);
                
                if (bytesRead > 0)
                {
        
                    state.sb.Remove(0, state.sb.Length); //clean the input from WinLirc
                    state.sb.Append(Encoding.ASCII.GetString(state.buffer, 0, bytesRead));
                    Logging.Log("Data: " + state.sb.ToString());
                    Logging.Log("Command: " + GetCommandName(state.sb.ToString(),false));
                    
                    //------------------------------
                    if (configmode == true)
                    {
                        //Loading the remote button into IRCommands and the listbox
                        if (GetCommandName(state.sb.ToString(), true) != "null")
                        {
                            if (wnd.learn == true)
                            {
                                IRcommands.Add(GetCommandName(state.sb.ToString(), true));
                                wnd.IRcommands.Items.Add(GetCommandName(state.sb.ToString(), true));
                            }
                        }
                        
                    }
                    else
                    {
                        //Normal Mode
                        string ReceivedCommandName = GetCommandName(state.sb.ToString(), true);
                        string ActionToSend = LoadCommandFromXML(ReceivedCommandName);
                        if (ActionToSend != "null")
                        {
                            //initiate MP action
                            Action act = new Action((Action.ActionType)Enum.Parse(typeof(Action.ActionType), ActionToSend), 0, 0);
                            GUIGraphicsContext.OnAction(act);
                        }
                        Logging.Log("ActionToSend: " + ActionToSend);
                        Logging.Log("Recieved command in normal mode");
                    }
                    //-------------------------------
                    client.BeginReceive(state.buffer, 0, StateObject.BufferSize, 0, new AsyncCallback(ReceiveCallback), state);
                    
                    
                }
                else
                {
                    // All the data has arrived; put it in response.
                    if (state.sb.Length > 1)
                    {
                        response = state.sb.ToString();
                        
                    }
                    
                    // Signal that all bytes have been received.
                    receiveDone.Set();
                }
            }
            catch 
            {
              
            }
        }

       

        private static void SendCallback(IAsyncResult ar)
        {
            try
            {
                // Retrieve the socket from the state object.
                Socket client = (Socket)ar.AsyncState;

                // Complete sending the data to the remote device.
                int bytesSent = client.EndSend(ar);

                // Signal that all bytes have been sent.
                sendDone.Set();
            }
            catch 
            {
               
            }
        }
        
        private static string GetCommandName(string Data, bool risingedge)
        {
            
                string[] content;
                content = Data.Split(' ');
                if (risingedge == true)
                {
                    if (content[1] == "00")
                    {
                        //Replace "/" with "-" because is it not allowed in a xml
                        return content[2];
                    }
                    else
                    {
                        //return "" when the button is still pressed
                        return "null";
                    }
                }
                else
                {
                    return content[2];
                }
            }

        private static void LoadMPCommands()
        {
            if (configmode == true)
            {
                
                MPcommands = Enum.GetValues(typeof(Action.ActionType));
                Logging.Log("MP commands loaded into mpcommands with " + MPcommands.Length.ToString() + " commands");
                for (int i = 0; i < MPcommands.Length; i++)
                {
                    wnd.MPCommands.Items.Add(MPcommands.GetValue(i).ToString());
                }
                wnd.MPCommands.Sorted = true;
                return;
            }
            else
            {
                MPcommands = Enum.GetValues(typeof(Action.ActionType));
                Logging.Log("MP commands loaded into mpcommands with " + MPcommands.Length.ToString() + " commands");
                return;
            }
        }

        public static void initCommandTable()
        {
            for (int i = 0; i < IRcommands.Count; i++)
            {
                CommandTable.Add("");
            }
        }

        public static void SaveToXML()
        {
            XmlWriterSettings settings = new XmlWriterSettings();
            settings.ConformanceLevel = ConformanceLevel.Document;
            settings.Indent = true;
            XmlWriter writer = XmlWriter.Create(Application.ExecutablePath.Remove(Application.ExecutablePath.LastIndexOf("\\")).ToString() + "\\plugins\\process\\WinLIRC_Plugin.xml", settings);

            writer.WriteStartElement("WINLIRC_PLUGIN_SETTINGS");
            for (int i = 0; i < Connection.IRcommands.Count; i++)
            {
                writer.WriteElementString("CMD" , IRcommands[i].ToString() + "|" + CommandTable[i].ToString());
            }
            writer.WriteEndElement();
            writer.Flush();
            writer.Close();
            Logging.Log("XML file has been saved");
        }

        public static string LoadCommandFromXML(string ReceivedCommand)
        {
            try
            {
                XmlElement root = doc.DocumentElement;
                XmlNodeList node = root.GetElementsByTagName("CMD");
                string[] content;
                for (int i = 0; i < node.Count; i++)
                {
                    content = node[i].InnerText.Split('|');
                    if (content[0] == ReceivedCommand)
                    {
                        if (content[1] == "")
                        {
                            break;
                        }
                        else
                        {
                            return content[1];
                        }
                    }
                }
                return "null";
                
            }
            catch
            {
                Logging.Log("Error while loading data from xml file");
                return "null";
            }
        }

        public static void LoadSettingsFromXML()
        {
            try
            {
                Logging.Log("Loading settings from xml:");
                //Load ircommands and define commandtable
                doc.Load(Application.ExecutablePath.Remove(Application.ExecutablePath.LastIndexOf("\\")).ToString() + "\\plugins\\process\\WinLIRC_Plugin.xml");
                XmlElement root = doc.DocumentElement;
                XmlNodeList node = root.GetElementsByTagName("CMD");
                Logging.Log("Commands in XML file:");
                CommandTable.Clear();
                IRcommands.Clear();
                wnd.IRcommands.Items.Clear();
                string[] content;
                for (int i = 0; i < node.Count; i++)
                {
                    content = node[i].InnerText.Split('|');
                    IRcommands.Add(content[0]);
                    CommandTable.Add(content[1]);
                    wnd.IRcommands.Items.Add(content[0]);
                    Logging.Log(content[0] + "  <-->  " + content[1]);
                } 
            }
            catch
            {
                Logging.Log("Error while loading settings from xml");
                return;
            }
        }
    }

}
